@@ -165,7 +165,7 @@ class Repository(object):
     svn = "svn"
     dev = []
 
-    def __init__(self, name, root=None, trunk=None, stable=None, release=None, tag=None, pyname=None, pypi=None, dev=False, username=None):
+    def __init__(self, name, root=None, trunk=None, stable=None, release=None, tag=None, pyname=None, pypi=None, dev=False, username=None, install=True, rev=None):
         class _TEMP_(object): pass
         self.config = _TEMP_()
         self.config.name=name
@@ -181,11 +181,16 @@ class Repository(object):
         else:
             self.config.dev=False
         self.config.username=username
+        if install == 'False' or install is False:
+            self.config.install=False
+        else:
+            self.config.install=True
+        self.config.rev=rev
         self.initialize(self.config)
 
     def initialize(self, config):
         self.name = config.name
-        self.root=config.root
+        self.root = config.root
         if not config.root is None:
             try:
                 self.trunk = config.root+'/trunk'
@@ -253,6 +258,13 @@ class Repository(object):
             self.svn_username = []
         else:
             self.svn_username = ['--username', config.username]
+        if config.rev is None:
+            self.rev=''
+            self.revarg=[]
+        else:
+            self.rev='@'+config.rev
+            self.revarg=['-r',config.rev]
+        self.install = config.install
 
     def write_config(self, OUTPUT):
         config = self.config
@@ -272,159 +284,124 @@ class Repository(object):
         elif not config.pyname is None:
             print >>OUTPUT, 'pypi=%s' % config.pyname
         print >>OUTPUT, 'dev=%s' % str(config.dev)
+        print >>OUTPUT, 'install=%s' % str(config.install)
+        if not config.rev is None:
+            print >>OUTPUT, 'rev=%s' % str(config.rev)
+        if not config.username is None:
+            print >>OUTPUT, 'username=%s' % str(config.username)
 
 
+    def find_pkgroot(self, trunk=False, stable=False, release=False):
+        if trunk:
+            if self.trunk is None:
+                if not self.stable is None:
+                    self.find_pkgroot(stable=True)
+                elif self.pypi is None:
+                    self.find_pkgroot(release=True)
+                else:
+                    # use easy_install
+                    self.pkgdir = None
+                    self.pkgroot = None
+                    return
+            else:
+                self.pkgdir = self.trunk
+                self.pkgroot = self.trunk_root
+                return
+
+        elif stable:
+            if self.stable is None: 
+                if not self.release is None:
+                    self.find_pkgroot(release=True)
+                elif self.pypi is None:
+                    self.find_pkgroot(trunk=True)
+                else:
+                    # use easy_install
+                    self.pkgdir = None
+                    self.pkgroot = None
+                    return
+            else:
+                self.pkgdir = self.stable
+                self.pkgroot = self.stable_root
+                return
+
+        elif release:
+            if self.release is None:
+                if not self.stable is None:
+                    self.find_pkgroot(stable=True)
+                elif self.pypi is None:
+                    self.find_pkgroot(trunk=True)
+                else:
+                    # use easy_install
+                    self.pkgdir = None
+                    self.pkgroot = None
+                    return
+            else:
+                self.pkgdir = self.release
+                self.pkgroot = self.release_root
+
+        else:
+            raise IOError, "Must have one of trunk, stable or release specified"
+            
+
     def install_trunk(self, dir=None, install=True, preinstall=False, offline=False):
-        if self.trunk is None:
-            if not self.stable is None:
-                self.install_stable(dir=dir, install=install)
-            elif self.pypi is None:
-                self.install_release(dir=dir, install=install)
-            else:
-                self.easy_install(install, preinstall, dir, offline)
-        else:
-            self.pkgdir=self.trunk
-            self.pkgroot=self.trunk_root
-            print "-----------------------------------------------------------------"
-            print "  Installing trunk branch"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.trunk
-            if os.path.exists(dir):
-                print "     No checkout required"
-                print "-----------------------------------------------------------------"
-            else:
-                print "-----------------------------------------------------------------"
-                self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.trunk, dir])
-            if install:
-                if self.dev:
-                    self.run([self.python, 'setup.py', 'develop'], dir=dir)
-                else:
-                    self.run([self.python, 'setup.py', 'install'], dir=dir)
-
+        self.find_pkgroot(trunk=True)
+        self.perform_install(dir=dir, install=install, preinstall=preinstall, offline=offline)
+        
     def install_stable(self, dir=None, install=True, preinstall=False, offline=False):
-        if self.stable is None: 
-            if not self.release is None:
-                self.install_release(dir=dir, install=install)
-            elif self.pypi is None:
-                self.install_trunk(dir=dir, install=install)
-            else:
-                self.easy_install(install, preinstall, dir, offline)
-        else:
-            self.pkgdir=self.stable
-            self.pkgroot=self.stable_root
-            print "-----------------------------------------------------------------"
-            print "  Installing stable branch"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.stable
-            print "     Source dir:     "+dir
-            if os.path.exists(dir):
-                print "     No checkout required"
-                print "-----------------------------------------------------------------"
-            else:
-                print "-----------------------------------------------------------------"
-                self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.stable, dir])
-            if install:
-                if self.dev:
-                    self.run([self.python, 'setup.py', 'develop'], dir=dir)
-                else:
-                    self.run([self.python, 'setup.py', 'install'], dir=dir)
-
+        self.find_pkgroot(stable=True)
+        self.perform_install(dir=dir, install=install, preinstall=preinstall, offline=offline)
+        
     def install_release(self, dir=None, install=True, preinstall=False, offline=False):
-        if self.release is None:
-            if not self.stable is None:
-                self.install_stable(dir=dir, install=install)
-            elif self.pypi is None:
-                self.install_trunk(dir=dir, install=install)
-            else:
-                self.easy_install(install, preinstall, dir, offline)
-        else:
-            self.pkgdir=self.release
-            self.pkgroot=self.release_root
+        self.find_pkgroot(release=True)
+        self.perform_install(dir=dir, install=install, preinstall=preinstall, offline=offline)
+        
+    def perform_install(self, dir=None, install=True, preinstall=False, offline=False):
+        if self.pkgdir is None:
+            self.easy_install(install, preinstall, dir, offline)
+            return
+        print "-----------------------------------------------------------------"
+        print "  Installing branch"
+        print "  Checking out source for package",self.name
+        print "     Subversion dir: "+self.pkgdir
+        if os.path.exists(dir):
+            print "     No checkout required"
             print "-----------------------------------------------------------------"
-            print "  Installing release branch"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.release
-            print "     Source dir:     "+dir
-            if os.path.exists(dir):
-                print "     No checkout required"
-                print "-----------------------------------------------------------------"
+        else:
+            print "-----------------------------------------------------------------"
+            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.pkgdir+self.rev, dir])
+        if install:
+            if self.dev:
+                self.run([self.python, 'setup.py', 'develop'], dir=dir)
             else:
-                print "-----------------------------------------------------------------"
-                self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.release, dir])
-            if install:
-                if self.dev:
-                    self.run([self.python, 'setup.py', 'develop'], dir=dir)
-                else:
-                    self.run([self.python, 'setup.py', 'install'], dir=dir)
+                self.run([self.python, 'setup.py', 'install'], dir=dir)
 
     def update_trunk(self, dir=None):
-        if self.trunk is None:
-            if not self.stable is None:
-                self.update_stable(dir=dir)
-            elif self.pypi is None:
-                self.update_release(dir=dir)
-            else:
-                self.easy_upgrade()
-        else:
-            self.pkgdir=self.trunk
-            self.pkgroot=self.trunk_root
-            print "-----------------------------------------------------------------"
-            print "  Updating trunk branch"
-            print "  Updating source for package",self.name
-            print "     Subversion dir: "+self.trunk
-            print "     Source dir:     "+dir
-            print "-----------------------------------------------------------------"
-            self.run([self.svn,'update','-q',dir])
-            if self.dev:
-                self.run([self.python, 'setup.py', 'develop'], dir=dir)
-            else:
-                self.run([self.python, 'setup.py', 'install'], dir=dir)
+        self.find_pkgroot(trunk=True)
+        self.perform_update(dir=dir)
 
     def update_stable(self, dir=None):
-        if self.stable is None:
-            if not self.release is None:
-                self.update_release(dir=dir)
-            elif self.pypi is None:
-                self.update_trunk(dir=dir)
-            else:
-                self.easy_upgrade()
-        else:
-            self.pkgdir=self.stable
-            self.pkgroot=self.stable_root
-            print "-----------------------------------------------------------------"
-            print "  Updating stable branch"
-            print "  Updating source for package",self.name
-            print "     Subversion dir: "+self.stable
-            print "     Source dir:     "+dir
-            print "-----------------------------------------------------------------"
-            self.run([self.svn,'update','-q',dir])
-            if self.dev:
-                self.run([self.python, 'setup.py', 'develop'], dir=dir)
-            else:
-                self.run([self.python, 'setup.py', 'install'], dir=dir)
+        self.find_pkgroot(stable=True)
+        self.perform_update(dir=dir)
 
     def update_release(self, dir=None):
-        if self.release is None:
-            if not self.stable is None:
-                self.update_stable(dir=dir)
-            elif self.pypi is None:
-                self.update_trunk(dir=dir)
-            else:
-                self.easy_upgrade()
+        self.find_pkgroot(release=True)
+        self.perform_update(dir=dir)
+
+    def perform_update(self, dir=None):
+        if self.pkgdir is None:
+            self.easy_upgrade()
+            return
+        print "-----------------------------------------------------------------"
+        print "  Updating branch"
+        print "  Updating source for package",self.name
+        print "     Subversion dir: "+self.pkgdir
+        print "     Source dir:     "+dir
+        print "-----------------------------------------------------------------"
+        self.run([self.svn,'update','-q']+self.revarg+[dir])
+        if self.dev:
+            self.run([self.python, 'setup.py', 'develop'], dir=dir)
         else:
-            self.pkgdir=self.release
-            self.pkgroot=self.release_root
-            print "-----------------------------------------------------------------"
-            print "  Updating release branch"
-            print "  Updating source for package",self.name
-            print "     Subversion dir: "+self.release
-            print "     Source dir:     "+dir
-            print "-----------------------------------------------------------------"
-            self.run([self.svn,'update','-q',dir])
-            if self.dev:
-                self.run([self.python, 'setup.py', 'develop'], dir=dir)
-            else:
-                self.run([self.python, 'setup.py', 'install'], dir=dir)
+            self.run([self.python, 'setup.py', 'install'], dir=dir)
 
     def Xsdist_trunk(self, format='zip'):
         if self.trunk is None:
@@ -441,7 +418,7 @@ class Repository(object):
             print "  Checking out source for package",self.name
             print "     Subversion dir: "+self.trunk
             print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.trunk, 'pkg'+self.name])
+            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.trunk, 'pkg'+self.name+self.rev])
             self.run([self.python, 'setup.py', 'sdist','-q','--dist-dir=..', '--formats='+format], dir='pkg'+self.name)
             os.chdir('..')
             rmtree('pkg'+self.name)
@@ -481,7 +458,7 @@ class Repository(object):
             print "     Subversion dir: "+self.release
             print "     Source dir:     "+dir
             print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.release, dir])
+            self.run([self.svn]+self.svn_username+[Repository.svn_get]+self.rev+['-q',self.release, dir])
             self.run([self.python, 'setup.py', 'install'], dir=dir)
 
     def easy_install(self, install, preinstall, dir, offline):
@@ -504,10 +481,11 @@ class Repository(object):
         self.run([self.easy_install_path, '-q', '--upgrade', self.pypi])
 
     def run(self, cmd, dir=None):
+        cwd=os.getcwd()
         if not dir is None:
-            cwd=os.getcwd()
             os.chdir(dir)
-        print "Running command '%s'" % " ".join(cmd)
+            cwd=dir
+        print "Running command '%s' in directory %s" % (" ".join(cmd), cwd)
         call_subprocess(cmd, filter_stdout=filter_python_develop, show_stdout=True)
         if not dir is None:
             os.chdir(cwd)
@@ -549,6 +527,7 @@ class Installer(object):
         self.cmd_files = []
         self.auxdir = []
         self.config=None
+        self.config_file=None
         self.README="""
 #
 # Virtual Python installation generated by the %s script.
@@ -636,18 +615,6 @@ class Installer(object):
             dest='zip',
             default=[])
 
-        parser.add_option('--forum-pkg',
-            help='Use one or more packages from the Coopr Forum.  Multiple packages are specified with a comma-separated list.',
-            action='store',
-            dest='forum',
-            default='')
-
-        parser.add_option('--forum-dev',
-            help="Explicitly indicate which of the Coopr Forum packages are treated as development packages.  By default, no Coopr Forum packages are treated this way; packages omited from this list are installed from their latest 'tags' branch.  Multiple packages are specified with a comma-separated list.",
-            action='store',
-            dest='forumdev',
-            default='')
-
         parser.add_option('--use-pythonpath',
             help="By default, the PYTHONPATH is ignored when installing.  This option allows the 'easy_install' tool to search this path for related Python packages, which are then installed.",
             action='store_true',
@@ -683,11 +650,11 @@ class Installer(object):
         # Change the virtualenv options
         #
         parser.get_option("--python").help = "Specify the Python interpreter to use, e.g., --python=python2.5 will install with the python2.5 interpreter."
-        parser.get_option("--clear").help = "Remove the existing installation and reinstall from scratch."
         parser.remove_option("--relocatable")
         parser.remove_option("--version")
         parser.remove_option("--unzip-setuptools")
         parser.remove_option("--no-site-packages")
+        parser.remove_option("--clear")
         #
         # Add description 
         #
@@ -696,6 +663,11 @@ class Installer(object):
         
 
     def adjust_options(self, options, args):
+        #
+        # Force options.clear to be False.  This allows us to preserve the logic
+        # associated with --clear, which we may want to use later.
+        #
+        options.clear=False
         #
         global vpy_main
         if options.debug:
@@ -716,7 +688,7 @@ class Installer(object):
             self.logger.fatal("ERROR: cannot specify --clear-config when specifying the --update option.")
             sys.exit(1000)
         if len(args) > 1:
-            self.logger.fatal("ERROR: "+self.installer.name+" can only have one argument")
+            self.logger.fatal("ERROR: installer script can only have one argument")
             sys.exit(1000)
         #
         # Error checking
@@ -725,8 +697,8 @@ class Installer(object):
             if options.update:
                 self.logger.fatal(wrapper.fill("ERROR: The 'update' option is specified, but the installation path '%s' does not exist!" % self.home_dir))
                 sys.exit(1000)
-            elif not options.clear and os.path.exists(join(self.abshome_dir,'bin')):
-                    self.logger.fatal(wrapper.fill("ERROR: The installation path '%s' already exists!  Use the --update option if you wish to update, or use --clear to create a fresh installation." % self.home_dir))
+            elif os.path.exists(join(self.abshome_dir,'bin')):
+                    self.logger.fatal(wrapper.fill("ERROR: The installation path '%s' already exists!  Use the --update option if you wish to update, or remove this directory to create a fresh installation." % self.home_dir))
                     sys.exit(1000)
         if len(args) == 0:
             args.append(self.abshome_dir)
@@ -740,6 +712,8 @@ class Installer(object):
             fp = StringIO.StringIO(self.config)
             self.read_config_file(fp=fp)
             fp.close()
+        if not self.config_file is None and not options.clear_config:
+            self.read_config_file(file=self.config_file)
         for file in options.config_files:
             self.read_config_file(file=file)
         print "-----------------------------------------------------------------"
@@ -789,7 +763,10 @@ class Installer(object):
         #
         # Setup HTTP proxy
         #
-        if not options.preinstall:
+        if options.offline:
+            os.environ['HTTP_PROXY'] = ''
+            os.environ['http_proxy'] = ''
+        else:
             proxy = ''
             if not options.proxy is None:
                 proxy = options.proxy
@@ -821,6 +798,13 @@ class Installer(object):
             rmtree(self.abshome_dir)
             os.mkdir(self.abshome_dir)
         #
+        # When preinstalling or working offline, disable the 
+        # default install_setuptools() function.
+        #
+        if options.preinstall or options.offline:
+            install_setuptools.use_default=False
+            install_pip.use_default=False
+        #
         # If we're clearing the current installation, then remove a bunch of
         # directories
         #
@@ -836,6 +820,10 @@ class Installer(object):
 
         if options.preinstall or not options.offline:
             self.get_packages(options)
+        else:
+            self.sw_packages.insert( 0, Repository('virtualenv', pypi='virtualenv') )
+            self.sw_packages.insert( 0, Repository('pip', pypi='pip') )
+            self.sw_packages.insert( 0, Repository('setuptools', pypi='setuptools') )
 
     def get_packages(self, options):
         #
@@ -868,37 +856,24 @@ class Installer(object):
         #
         # Get source packages
         #
+        self.sw_packages.insert( 0, Repository('virtualenv', pypi='virtualenv') )
+        self.sw_packages.insert( 0, Repository('pip', pypi='pip') )
         if options.preinstall:
             #
-            # When preinstalling, disable the default install_setuptools() function,
-            # and add the setuptools package to the installation list
+            # When preinstalling, add the setuptools package to the installation list
             #
-            install_setuptools.use_default=False
             self.sw_packages.insert( 0, Repository('setuptools', pypi='setuptools') )
         #
         # Add Coopr Forum packages
         #
-        dev = options.forumdev.split(',')
-        for pkg in options.forum.split(','):
-            if pkg is '':
-                continue
-            if pkg in dev:
-                if (options.trunk or options.stable):
-                    self.sw_packages[pkg] = 'http://coopr-forum.googlecode.com/svn/'+pkg
-                else:
-                    self.sw_packages[pkg] = '-f http://coopr-forum.googlecode.com/svn/'+pkg+'/trunk '+pkg
-                dev_packages.append(pkg)
-            else:
-                try:
-                    self.sw_packages[pkg] = guess_release('http://coopr-forum.googlecode.com/svn/'+pkg+'/dev/')
-                except Exception, err:
-                    print "-----------------------------------------------------------------"
-                    print "ERROR Finding 'tags' branch for Coopr Forum Package %s: %s" % (pkg,str(err))
-                    print "-----------------------------------------------------------------"
+        self.get_other_packages(options)
         #
         # Get package source
         #
         for pkg in self.sw_packages:
+            if not pkg.install:
+                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=not (options.trunk or options.stable))
+                continue
             if pkg.dev:
                 tmp = join(self.abshome_dir,'src',pkg.name)
             else:
@@ -931,6 +906,12 @@ class Installer(object):
             zip_file(self.default_dirname+'.zip', ['.'])
             sys.exit(0)
 
+    def get_other_packages(self, options):
+        #
+        # Used by subclasses of Installer to 
+        # add packages that were requested through other means....
+        #
+        pass
         
     def install_packages(self, options):
         #
@@ -946,6 +927,9 @@ class Installer(object):
         # Install the related packages
         #
         for pkg in self.sw_packages:
+            if not pkg.install:
+                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=not (options.trunk or options.stable))
+                continue
             if pkg.dev:
                 srcdir = join(self.abshome_dir,'src',pkg.name)
             else:
@@ -1028,7 +1012,7 @@ class Installer(object):
             try:
                 output = urllib2.urlopen(file).read()
             except Exception, err:
-                print "Problems opening configuration url:",url
+                print "Problems opening configuration url:",file
                 raise
             fp = StringIO.StringIO(output)
             parser.readfp(fp, file)
@@ -1046,8 +1030,8 @@ class Installer(object):
         for sec in sections:
             if sec in ['installer', 'localize']:
                 continue
-            if sec.startswith('auxdir_'):
-                auxdir = sec[7:]
+            if sec.endswith(':auxdir'):
+                auxdir = sec[:-7]
                 for option, value in parser.items(sec):
                     self.add_auxdir(auxdir, option, apply_template(value, os.environ) )
             else:
@@ -1159,6 +1143,28 @@ def install_setuptools(py_executable, unzip=False):
 install_setuptools.use_default=True
 
 
+#
+# This is a monkey patch, to control the execution of the install_pip()
+# function that is defined by virtualenv.
+#
+default_install_pip = install_pip
+
+def install_pip(*args, **kwds):
+    try:
+        if install_pip.use_default:
+            default_install_pip(*args, **kwds)
+    except OSError, err:
+        print "-----------------------------------------------------------------"
+        print "Error installing the 'pip' package!"
+        if os.environ['HTTP_PROXY'] == '':
+            print ""
+            print "WARNING: you may need to set your HTTP_PROXY environment variable!"
+        print "-----------------------------------------------------------------"
+        sys.exit(0)
+
+install_pip.use_default=True
+
+
 #
 # The following methods will be called by virtualenv
 #
